from threading import Thread
from pbesa.kernel.system.Adm import Adm
from pbesa.kernel.adapter.Adapter import Adapter
from pbesa.middleware.adapter.WSSNJHandler import WSSNJHandler
from pbesa.middleware.adapter.SubProcessAdapter import SubProcessAdapter
from pbesa.kernel.io.TCPServer import TCPServer

import os
import sys
import json
import socketserver

import threading

class WSSNJAdapter(AdapTK, Thread):

    IP = None
    adm = None
    PORT = None
    server = None

    def __init__(self):
        self.adm = Adm()
        conf = self.adm.getConf()
        self.IP = conf['ip']
        self.PORT = conf['port']
        self.INTERNAL_PORT = conf['internal_port']
        super().__init__()

    def setUp(self):
        DIR = os.path.dirname(os.path.abspath(__file__)) 
        FILE = os.path.join(DIR,"WSSNJWrapper.js")
        spa = SubProcessAdapter("node", FILE, self.IP + '-' + str(self.PORT) + '-' + str(self.INTERNAL_PORT))
        spa.start()
        
    def response(self, response):
        pass
    
    def request(self):
        pass

    def finalize(self):
        self.server.shutdown()
        self.server.server_close()        

    def run(self):
        self.server = TCPServerTK((self.IP, self.INTERNAL_PORT), WSSNJHandlerTK)
        server_thread = threading.Thread(target=self.server.serve_forever)
        server_thread.daemon = True
        server_thread.start()




from threading import Thread
from abc import ABC, abstractmethod
from pbesa.Kernel.system.Adm import Adm
from pbesa.Kernel.adapter.Adapter import Adapter

import os
import random
import pygame
import math
import sys

class GameAdapter(AdapTK, Thread):

    adm = None
    conf = None
    clock = None
    screen = None
    agentList = None

    def __init__(self, conf):        
        self.conf = conf
        self.agentList = []
        self.adm = Adm()
        super().__init__()

    def setUp(self):
        os.environ["SDL_VIDEO_CENTERED"] = "1"
        self.screen = pygame.display.set_mode((self.conf['width'], self.conf['height']))
        pygame.display.set_caption(self.conf['title'])
        self.clock = pygame.time.Clock()
        pygame.init()
        self.clock = pygame.time.Clock()

    def response(self):
        pass
    
    def request(self):
        return self.data

    def finalize(self):
        pass

    def addAgent(self, ag):
        self.agentList.append(ag)

    def run(self):
        running = True       
        while running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    running = False
            self.frame()
            pygame.display.update()
            self.clock.tick(40)

    def updateWorlds(self, event, data):
        for ag in self.agentList:
            self.adm.sendEvent(ag.id, event, data)

    def getPygame(self):
        return pygame

    def frame(self):
        pass


import pymongo

from pymongo import MongoClient
from pbesa.kernel.adapter.Adapter import Adapter

class MongoAdapter(AdapTK):

    db = None
    conf = None
    client = None

    def __init__(self, conf):
        self.conf = conf
        super().__init__()

    def setUp(self):
        self.client = MongoClient()
        self.db = self.client[self.conf['database']]

    def response(self, response):
        pass
    
    def request(self):
        pass
        
    def finalize(self):
        pass

    def getAll(self, collection):
        coll = self.db[collection]
        return coll.find()

    def getObject(self, collection, query):
        coll = self.db[collection]
        return coll.find_one(query)

    def getObjectByID(self, collection, id):
        coll = self.db[collection]
        return coll.find_one({"_id": id})

    def insertObject(self, collection, dto):
        coll = self.db[collection]
        return coll.insert_one(dto).inserted_id

    def updateObject(self, collection, id, dto):
        coll = self.db[collection]
        return coll.replace_one({"_id": id}, dto, True)
